@@ -1,10 +1,10 @@
 //     __ _____ _____ _____
 //  __|  |   __|     |   | |  JSON for Modern C++ (supporting code)
-// |  |  |__   |  |  | | | |  version 3.11.1
+// |  |  |__   |  |  | | | |  version 3.11.3
 // |_____|_____|_____|_|___|  https://github.com/nlohmann/json
 //
 // Copyright (c) 2013-2022 Niels Lohmann <http://nlohmann.me>.
-// SPDX-FileCopyrightText: 2013-2022 Niels Lohmann <https://nlohmann.me>
+// SPDX-FileCopyrightText: 2013-2023 Niels Lohmann <https://nlohmann.me>
 // SPDX-License-Identifier: MIT
 
 #include "doctest_compatibility.h"
@@ -14,6 +14,9 @@
     using namespace nlohmann::literals; // NOLINT(google-build-using-namespace)
 #endif
 
+// build test with C++14
+// JSON_HAS_CPP_14
+
 TEST_CASE_TEMPLATE("element access 2", Json, nlohmann::json, nlohmann::ordered_json)
 {
     SECTION("object")
@@ -69,7 +72,6 @@ TEST_CASE_TEMPLATE("element access 2", Json, nlohmann::json, nlohmann::ordered_j
                 CHECK_THROWS_WITH_AS(j.at("foo"), "[json.exception.out_of_range.403] key 'foo' not found", typename Json::out_of_range&);
                 CHECK_THROWS_WITH_AS(j_const.at("foo"), "[json.exception.out_of_range.403] key 'foo' not found", typename Json::out_of_range&);
 
-
 #ifdef JSON_HAS_CPP_17
                 CHECK_THROWS_WITH_AS(j.at(std::string_view("foo")), "[json.exception.out_of_range.403] key 'foo' not found", typename Json::out_of_range&);
                 CHECK_THROWS_WITH_AS(j_const.at(std::string_view("foo")), "[json.exception.out_of_range.403] key 'foo' not found", typename Json::out_of_range&);
@@ -466,7 +468,7 @@ TEST_CASE_TEMPLATE("element access 2", Json, nlohmann::json, nlohmann::ordered_j
             }
 #ifdef JSON_HAS_CPP_17
             {
-                std::string_view key = "key";
+                std::string_view const key = "key";
                 Json j_null;
                 CHECK(j_null.is_null());
                 j_null[key] = 1;
@@ -830,13 +832,13 @@ TEST_CASE_TEMPLATE("element access 2", Json, nlohmann::json, nlohmann::ordered_j
                 {
                     {
                         Json jobject = {{"a", "a"}, {"b", 1}, {"c", 17u}};
-                        typename Json::iterator it2 = jobject.erase(jobject.begin());
+                        typename Json::iterator const it2 = jobject.erase(jobject.begin());
                         CHECK(jobject == Json({{"b", 1}, {"c", 17u}}));
                         CHECK(*it2 == Json(1));
                     }
                     {
                         Json jobject = {{"a", "a"}, {"b", 1}, {"c", 17u}};
-                        typename Json::const_iterator it2 = jobject.erase(jobject.cbegin());
+                        typename Json::const_iterator const it2 = jobject.erase(jobject.cbegin());
                         CHECK(jobject == Json({{"b", 1}, {"c", 17u}}));
                         CHECK(*it2 == Json(1));
                     }
@@ -862,13 +864,13 @@ TEST_CASE_TEMPLATE("element access 2", Json, nlohmann::json, nlohmann::ordered_j
                 {
                     {
                         Json jobject = {{"a", "a"}, {"b", 1}, {"c", 17u}};
-                        typename Json::iterator it2 = jobject.erase(jobject.begin(), jobject.begin());
+                        typename Json::iterator const it2 = jobject.erase(jobject.begin(), jobject.begin());
                         CHECK(jobject == Json({{"a", "a"}, {"b", 1}, {"c", 17u}}));
                         CHECK(*it2 == Json("a"));
                     }
                     {
                         Json jobject = {{"a", "a"}, {"b", 1}, {"c", 17u}};
-                        typename Json::const_iterator it2 = jobject.erase(jobject.cbegin(), jobject.cbegin());
+                        typename Json::const_iterator const it2 = jobject.erase(jobject.cbegin(), jobject.cbegin());
                         CHECK(jobject == Json({{"a", "a"}, {"b", 1}, {"c", 17u}}));
                         CHECK(*it2 == Json("a"));
                     }
@@ -878,15 +880,15 @@ TEST_CASE_TEMPLATE("element access 2", Json, nlohmann::json, nlohmann::ordered_j
                 {
                     {
                         Json jobject = {{"a", "a"}, {"b", 1}, {"c", 17u}};
-                        typename Json::iterator it = jobject.find("b");
-                        typename Json::iterator it2 = jobject.erase(it);
+                        typename Json::iterator const it = jobject.find("b");
+                        typename Json::iterator const it2 = jobject.erase(it);
                         CHECK(jobject == Json({{"a", "a"}, {"c", 17u}}));
                         CHECK(*it2 == Json(17));
                     }
                     {
                         Json jobject = {{"a", "a"}, {"b", 1}, {"c", 17u}};
-                        typename Json::const_iterator it = jobject.find("b");
-                        typename Json::const_iterator it2 = jobject.erase(it);
+                        typename Json::const_iterator const it = jobject.find("b");
+                        typename Json::const_iterator const it2 = jobject.erase(it);
                         CHECK(jobject == Json({{"a", "a"}, {"c", 17u}}));
                         CHECK(*it2 == Json(17));
                     }
@@ -896,13 +898,13 @@ TEST_CASE_TEMPLATE("element access 2", Json, nlohmann::json, nlohmann::ordered_j
                 {
                     {
                         Json jobject = {{"a", "a"}, {"b", 1}, {"c", 17u}, {"d", false}, {"e", true}};
-                        typename Json::iterator it2 = jobject.erase(jobject.find("b"), jobject.find("e"));
+                        typename Json::iterator const it2 = jobject.erase(jobject.find("b"), jobject.find("e"));
                         CHECK(jobject == Json({{"a", "a"}, {"e", true}}));
                         CHECK(*it2 == Json(true));
                     }
                     {
                         Json jobject = {{"a", "a"}, {"b", 1}, {"c", 17u}, {"d", false}, {"e", true}};
-                        typename Json::const_iterator it2 = jobject.erase(jobject.find("b"), jobject.find("e"));
+                        typename Json::const_iterator const it2 = jobject.erase(jobject.find("b"), jobject.find("e"));
                         CHECK(jobject == Json({{"a", "a"}, {"e", true}}));
                         CHECK(*it2 == Json(true));
                     }
@@ -1488,3 +1490,304 @@ TEST_CASE_TEMPLATE("element access 2 (throwing tests)", Json, nlohmann::json, nl
     }
 }
 #endif
+
+// TODO(falbrechtskirchinger) merge with the other test case; clean up
+TEST_CASE_TEMPLATE("element access 2 (additional value() tests)", Json, nlohmann::json, nlohmann::ordered_json)
+{
+    using string_t = typename Json::string_t;
+    using number_integer_t = typename Json::number_integer_t;
+
+    // test assumes string_t and object_t::key_type are the same
+    REQUIRE(std::is_same<string_t, typename Json::object_t::key_type>::value);
+
+    Json j
+    {
+        {"foo", "bar"},
+        {"baz", 42}
+    };
+
+    const char* cpstr = "default";
+    const char castr[] = "default"; // NOLINT(hicpp-avoid-c-arrays,modernize-avoid-c-arrays,cppcoreguidelines-avoid-c-arrays)
+    string_t const str = "default";
+
+    number_integer_t integer = 69;
+    std::size_t size = 69;
+
+    SECTION("deduced ValueType")
+    {
+        SECTION("literal key")
+        {
+            CHECK(j.value("foo", "default") == "bar");
+            CHECK(j.value("foo", cpstr) == "bar");
+            CHECK(j.value("foo", castr) == "bar");
+            CHECK(j.value("foo", str) == "bar");
+            // this test is in fact different from the one below,
+            // because of 0 considering const char * overloads
+            // whereas any other number does not
+            CHECK(j.value("baz", 0) == 42);
+            CHECK(j.value("baz", 47) == 42);
+            CHECK(j.value("baz", integer) == 42);
+            CHECK(j.value("baz", size) == 42);
+
+            CHECK(j.value("bar", "default") == "default");
+            CHECK(j.value("bar", 0) == 0);
+            CHECK(j.value("bar", 47) == 47);
+            CHECK(j.value("bar", integer) == integer);
+            CHECK(j.value("bar", size) == size);
+
+            CHECK_THROWS_WITH_AS(Json().value("foo", "default"), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+            CHECK_THROWS_WITH_AS(Json().value("foo", str), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+        }
+
+        SECTION("const char * key")
+        {
+            const char* key = "foo";
+            const char* key2 = "baz";
+            const char* key_notfound = "bar";
+
+            CHECK(j.value(key, "default") == "bar");
+            CHECK(j.value(key, cpstr) == "bar");
+            CHECK(j.value(key, castr) == "bar");
+            CHECK(j.value(key, str) == "bar");
+            CHECK(j.value(key2, 0) == 42);
+            CHECK(j.value(key2, 47) == 42);
+            CHECK(j.value(key2, integer) == 42);
+            CHECK(j.value(key2, size) == 42);
+
+            CHECK(j.value(key_notfound, "default") == "default");
+            CHECK(j.value(key_notfound, 0) == 0);
+            CHECK(j.value(key_notfound, 47) == 47);
+            CHECK(j.value(key_notfound, integer) == integer);
+            CHECK(j.value(key_notfound, size) == size);
+
+            CHECK_THROWS_WITH_AS(Json().value(key, "default"), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+            CHECK_THROWS_WITH_AS(Json().value(key, str), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+        }
+
+        SECTION("const char(&)[] key")
+        {
+            const char key[] = "foo"; // NOLINT(hicpp-avoid-c-arrays,modernize-avoid-c-arrays,cppcoreguidelines-avoid-c-arrays)
+            const char key2[] = "baz"; // NOLINT(hicpp-avoid-c-arrays,modernize-avoid-c-arrays,cppcoreguidelines-avoid-c-arrays)
+            const char key_notfound[] = "bar"; // NOLINT(hicpp-avoid-c-arrays,modernize-avoid-c-arrays,cppcoreguidelines-avoid-c-arrays)
+
+            CHECK(j.value(key, "default") == "bar");
+            CHECK(j.value(key, cpstr) == "bar");
+            CHECK(j.value(key, castr) == "bar");
+            CHECK(j.value(key, str) == "bar");
+            CHECK(j.value(key2, 0) == 42);
+            CHECK(j.value(key2, 47) == 42);
+            CHECK(j.value(key2, integer) == 42);
+            CHECK(j.value(key2, size) == 42);
+
+            CHECK(j.value(key_notfound, "default") == "default");
+            CHECK(j.value(key_notfound, 0) == 0);
+            CHECK(j.value(key_notfound, 47) == 47);
+            CHECK(j.value(key_notfound, integer) == integer);
+            CHECK(j.value(key_notfound, size) == size);
+
+            CHECK_THROWS_WITH_AS(Json().value(key, "default"), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+            CHECK_THROWS_WITH_AS(Json().value(key, str), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+        }
+
+        SECTION("string_t/object_t::key_type key")
+        {
+            string_t const key = "foo";
+            string_t const key2 = "baz";
+            string_t const key_notfound = "bar";
+
+            CHECK(j.value(key, "default") == "bar");
+            CHECK(j.value(key, cpstr) == "bar");
+            CHECK(j.value(key, castr) == "bar");
+            CHECK(j.value(key, str) == "bar");
+            CHECK(j.value(key2, 0) == 42);
+            CHECK(j.value(key2, 47) == 42);
+            CHECK(j.value(key2, integer) == 42);
+            CHECK(j.value(key2, size) == 42);
+
+            CHECK(j.value(key_notfound, "default") == "default");
+            CHECK(j.value(key_notfound, 0) == 0);
+            CHECK(j.value(key_notfound, 47) == 47);
+            CHECK(j.value(key_notfound, integer) == integer);
+            CHECK(j.value(key_notfound, size) == size);
+
+            CHECK_THROWS_WITH_AS(Json().value(key, "default"), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+            CHECK_THROWS_WITH_AS(Json().value(key, str), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+        }
+
+#ifdef JSON_HAS_CPP_17
+        SECTION("std::string_view key")
+        {
+            std::string_view const key = "foo";
+            std::string_view const key2 = "baz";
+            std::string_view const key_notfound = "bar";
+
+            CHECK(j.value(key, "default") == "bar");
+            CHECK(j.value(key, cpstr) == "bar");
+            CHECK(j.value(key, castr) == "bar");
+            CHECK(j.value(key, str) == "bar");
+            CHECK(j.value(key2, 0) == 42);
+            CHECK(j.value(key2, 47) == 42);
+            CHECK(j.value(key2, integer) == 42);
+            CHECK(j.value(key2, size) == 42);
+
+            CHECK(j.value(key_notfound, "default") == "default");
+            CHECK(j.value(key_notfound, 0) == 0);
+            CHECK(j.value(key_notfound, 47) == 47);
+            CHECK(j.value(key_notfound, integer) == integer);
+            CHECK(j.value(key_notfound, size) == size);
+
+            CHECK_THROWS_WITH_AS(Json().value(key, "default"), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+            CHECK_THROWS_WITH_AS(Json().value(key, str), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+        }
+#endif
+    }
+
+    SECTION("explicit ValueType")
+    {
+        SECTION("literal key")
+        {
+            CHECK(j.template value<string_t>("foo", "default") == "bar");
+            CHECK(j.template value<string_t>("foo", cpstr) == "bar");
+            CHECK(j.template value<string_t>("foo", castr) == "bar");
+            CHECK(j.template value<string_t>("foo", str) == "bar");
+            CHECK(j.template value<number_integer_t>("baz", 0) == 42);
+            CHECK(j.template value<number_integer_t>("baz", 47) == 42);
+            CHECK(j.template value<number_integer_t>("baz", integer) == 42);
+            CHECK(j.template value<std::size_t>("baz", 0) == 42);
+            CHECK(j.template value<std::size_t>("baz", 47) == 42);
+            CHECK(j.template value<std::size_t>("baz", size) == 42);
+
+            CHECK(j.template value<string_t>("bar", "default") == "default");
+            CHECK(j.template value<number_integer_t>("bar", 0) == 0);
+            CHECK(j.template value<number_integer_t>("bar", 47) == 47);
+            CHECK(j.template value<number_integer_t>("bar", integer) == integer);
+            CHECK(j.template value<std::size_t>("bar", 0) == 0);
+            CHECK(j.template value<std::size_t>("bar", 47) == 47);
+            CHECK(j.template value<std::size_t>("bar", size) == size);
+
+            CHECK_THROWS_WITH_AS(Json().template value<string_t>("foo", "default"), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+            CHECK_THROWS_WITH_AS(Json().template value<string_t>("foo", str), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+        }
+
+        SECTION("const char * key")
+        {
+            const char* key = "foo";
+            const char* key2 = "baz";
+            const char* key_notfound = "bar";
+
+            CHECK(j.template value<string_t>(key, "default") == "bar");
+            CHECK(j.template value<string_t>(key, cpstr) == "bar");
+            CHECK(j.template value<string_t>(key, castr) == "bar");
+            CHECK(j.template value<string_t>(key, str) == "bar");
+            CHECK(j.template value<number_integer_t>(key2, 0) == 42);
+            CHECK(j.template value<number_integer_t>(key2, 47) == 42);
+            CHECK(j.template value<number_integer_t>(key2, integer) == 42);
+            CHECK(j.template value<std::size_t>(key2, 0) == 42);
+            CHECK(j.template value<std::size_t>(key2, 47) == 42);
+            CHECK(j.template value<std::size_t>(key2, size) == 42);
+
+            CHECK(j.template value<string_t>(key_notfound, "default") == "default");
+            CHECK(j.template value<number_integer_t>(key_notfound, 0) == 0);
+            CHECK(j.template value<number_integer_t>(key_notfound, 47) == 47);
+            CHECK(j.template value<number_integer_t>(key_notfound, integer) == integer);
+            CHECK(j.template value<std::size_t>(key_notfound, 0) == 0);
+            CHECK(j.template value<std::size_t>(key_notfound, 47) == 47);
+            CHECK(j.template value<std::size_t>(key_notfound, size) == size);
+
+            CHECK_THROWS_WITH_AS(Json().template value<string_t>(key, "default"), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+            CHECK_THROWS_WITH_AS(Json().template value<string_t>(key, str), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+        }
+
+        SECTION("const char(&)[] key")
+        {
+            const char key[] = "foo"; // NOLINT(hicpp-avoid-c-arrays,modernize-avoid-c-arrays,cppcoreguidelines-avoid-c-arrays)
+            const char key2[] = "baz"; // NOLINT(hicpp-avoid-c-arrays,modernize-avoid-c-arrays,cppcoreguidelines-avoid-c-arrays)
+            const char key_notfound[] = "bar"; // NOLINT(hicpp-avoid-c-arrays,modernize-avoid-c-arrays,cppcoreguidelines-avoid-c-arrays)
+
+            CHECK(j.template value<string_t>(key, "default") == "bar");
+            CHECK(j.template value<string_t>(key, cpstr) == "bar");
+            CHECK(j.template value<string_t>(key, castr) == "bar");
+            CHECK(j.template value<string_t>(key, str) == "bar");
+            CHECK(j.template value<number_integer_t>(key2, 0) == 42);
+            CHECK(j.template value<number_integer_t>(key2, 47) == 42);
+            CHECK(j.template value<number_integer_t>(key2, integer) == 42);
+            CHECK(j.template value<std::size_t>(key2, 0) == 42);
+            CHECK(j.template value<std::size_t>(key2, 47) == 42);
+            CHECK(j.template value<std::size_t>(key2, size) == 42);
+
+            CHECK(j.template value<string_t>(key_notfound, "default") == "default");
+            CHECK(j.template value<number_integer_t>(key_notfound, 0) == 0);
+            CHECK(j.template value<number_integer_t>(key_notfound, 47) == 47);
+            CHECK(j.template value<number_integer_t>(key_notfound, integer) == integer);
+            CHECK(j.template value<std::size_t>(key_notfound, 0) == 0);
+            CHECK(j.template value<std::size_t>(key_notfound, 47) == 47);
+            CHECK(j.template value<std::size_t>(key_notfound, size) == size);
+
+            CHECK_THROWS_WITH_AS(Json().template value<string_t>(key, "default"), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+            CHECK_THROWS_WITH_AS(Json().template value<string_t>(key, str), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+        }
+
+        SECTION("string_t/object_t::key_type key")
+        {
+            string_t const key = "foo";
+            string_t const key2 = "baz";
+            string_t const key_notfound = "bar";
+
+            CHECK(j.template value<string_t>(key, "default") == "bar");
+            CHECK(j.template value<string_t>(key, cpstr) == "bar");
+            CHECK(j.template value<string_t>(key, castr) == "bar");
+            CHECK(j.template value<string_t>(key, str) == "bar");
+            CHECK(j.template value<number_integer_t>(key2, 0) == 42);
+            CHECK(j.template value<number_integer_t>(key2, 47) == 42);
+            CHECK(j.template value<std::size_t>(key2, 0) == 42);
+            CHECK(j.template value<std::size_t>(key2, 47) == 42);
+
+            CHECK(j.template value<string_t>(key_notfound, "default") == "default");
+            CHECK(j.template value<number_integer_t>(key_notfound, 0) == 0);
+            CHECK(j.template value<number_integer_t>(key_notfound, 47) == 47);
+            CHECK(j.template value<std::size_t>(key_notfound, 0) == 0);
+            CHECK(j.template value<std::size_t>(key_notfound, 47) == 47);
+
+            CHECK_THROWS_WITH_AS(Json().template value<string_t>(key, "default"), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+            CHECK_THROWS_WITH_AS(Json().template value<string_t>(key, str), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+        }
+
+#ifdef JSON_HAS_CPP_17
+        SECTION("std::string_view key")
+        {
+            std::string_view const key = "foo";
+            std::string_view const key2 = "baz";
+            std::string_view const key_notfound = "bar";
+
+            CHECK(j.template value<string_t>(key, "default") == "bar");
+            CHECK(j.template value<string_t>(key, cpstr) == "bar");
+            CHECK(j.template value<string_t>(key, castr) == "bar");
+            CHECK(j.template value<string_t>(key, str) == "bar");
+            CHECK(j.template value<number_integer_t>(key2, 0) == 42);
+            CHECK(j.template value<number_integer_t>(key2, 47) == 42);
+            CHECK(j.template value<number_integer_t>(key2, integer) == 42);
+            CHECK(j.template value<std::size_t>(key2, 0) == 42);
+            CHECK(j.template value<std::size_t>(key2, 47) == 42);
+            CHECK(j.template value<std::size_t>(key2, size) == 42);
+
+            CHECK(j.template value<string_t>(key_notfound, "default") == "default");
+            CHECK(j.template value<number_integer_t>(key_notfound, 0) == 0);
+            CHECK(j.template value<number_integer_t>(key_notfound, 47) == 47);
+            CHECK(j.template value<number_integer_t>(key_notfound, integer) == integer);
+            CHECK(j.template value<std::size_t>(key_notfound, 0) == 0);
+            CHECK(j.template value<std::size_t>(key_notfound, 47) == 47);
+            CHECK(j.template value<std::size_t>(key_notfound, size) == size);
+
+            CHECK(j.template value<std::string_view>(key, "default") == "bar");
+            CHECK(j.template value<std::string_view>(key, cpstr) == "bar");
+            CHECK(j.template value<std::string_view>(key, castr) == "bar");
+            CHECK(j.template value<std::string_view>(key, str) == "bar");
+
+            CHECK(j.template value<std::string_view>(key_notfound, "default") == "default");
+
+            CHECK_THROWS_WITH_AS(Json().template value<string_t>(key, "default"), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+            CHECK_THROWS_WITH_AS(Json().template value<string_t>(key, str), "[json.exception.type_error.306] cannot use value() with null", typename Json::type_error&);
+        }
+#endif
+    }
+}
